# Copyright 2017 Jairo Llopis <jairo.llopis@tecnativa.com>
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).
from odoo.tests import new_test_user, tagged
from odoo.tests.common import HttpCase


@tagged("post_install", "-at_install")
class UICase(HttpCase):
    """Test checkout flow with legal terms acceptance required.

    It would be nice to check also that the workflow isn't interrupted when the
    acceptance requirement views are disabled, but that's what upstream tests
    do, so we don't need to repeat them. We can assume that, if other tests in
    the same integrated environment don't fail because of lack of legal terms
    acceptance, then the flow is fine.
    """

    def setUp(self):
        """Ensure website lang is en_US."""
        super().setUp()
        website = self.env["website"].get_current_website()
        en_US = (
            self.env["res.lang"]
            .with_context(active_test=False)
            .search([("code", "=", "en_US")])
        )
        wiz = self.env["base.language.install"].create({"lang_ids": en_US.ids})
        self.env.flush_all()
        wiz.website_ids = website
        wiz.lang_install()
        website.default_lang_id = self.env.ref("base.lang_en")
        # Activate Accept Terms & Conditions views, as explained in CONFIGURE.rst
        website.viewref(
            "website_sale_require_legal.address_require_legal"
        ).active = True
        website.viewref("website_sale.payment_sale_note").active = True
        new_test_user(self.env, login="super_mario", groups="base.group_portal")
        # Create a dummy payment provider to ensure that the tour has at least one
        # available to it.
        arch = """
        <form action="dummy" method="post">
            <input type="hidden" name="view_id" t-att-value="viewid"/>
            <input type="hidden" name="user_id" t-att-value="user_id.id"/>
        </form>
        """
        redirect_form = self.env["ir.ui.view"].create(
            {
                "name": "Dummy Redirect Form",
                "type": "qweb",
                "arch": arch,
            }
        )
        self.dummy_provider = self.env["payment.provider"].create(
            {
                "name": "Dummy Provider",
                "code": "none",
                "state": "test",
                "is_published": True,
                "allow_tokenization": True,
                "redirect_form_view_id": redirect_form.id,
            }
        )

    def test_ui_website(self):
        """Test frontend tour."""
        self.start_tour("/shop", "website_sale_require_legal", login="super_mario")
        order = self.env["sale.order"].search(
            [
                ("partner_id", "ilike", "super_mario"),
                ("website_id", "!=", "False"),
            ]
        )
        partner = order.partner_id
        # Assert that the sale order and partner have metadata logs
        self.assertTrue(
            order.message_ids.filtered(
                lambda one: "Website legal terms acceptance metadata" in one.body
            )
        )
        self.assertTrue(
            partner.message_ids.filtered(
                lambda one: "Website legal terms acceptance metadata" in one.body
            )
        )
